/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2000 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Release: 3.0
 *
 * Contact: jonathan@objectweb.org
 *
 * Author: Fabien Delpiano, Bruno Dumant
 * 
 */

package org.objectweb.jonathan.apis.kernel;

/** 
 * <code>Factory</code> represents the type for factories of bootstrap services. 
 * <p>
 * Factories are used when it is not possible to describe directly an element in the
 * Kernel.xml configuration file. In this case, factories may be used. A factory should
 * implement the {@link Factory Factory} interface and have a public no-arg constructor.
 * The factory is registered in the configuration file using a <code>FACTORY</code>
 * construct stipulating the name of the context element to construct and the 
 * java class name of the factory.
 */
public interface Factory{

   /** 
    * Returns an object created using information contained in the provided 
    * {@link Context context}.
    * @param context a {@link Context Context} object;
    * @return an object created using information contained in <code>context</code>.
    * @exception JonathanException if something goes wrong.
    */
   Object newObject(Context c) throws JonathanException;
}
