/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 *
 * -------------------------------------------------------------------
 * $Id: Initializer.java,v 1.1.1.1 2002/08/06 13:13:47 bmi Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.apis.kernel;

/** 
 * <code>Initializer</code> represents the type for initializers of services. 
 * <p>
 * Initializers are used in order to break circularities while creating services.
 * For the time being, the only way to use initializers is to use factories
 * that also implement the  {@link Initializer Initializer} interface. 
 */
public interface Initializer {

   /** 
    * Initialize an object using information contained in the provided 
    * {@link Context context}.
    * @param object initialized using information contained in <code>context</code>.
    * @param context a {@link Context Context} object;
    * @param alternative the implementation alternative to use. If there is no
    * alternative, this argument's value is -1.
    * @exception JonathanException if something goes wrong.
    */
   public void initialize(Object object,Context context,int alternative) throws JonathanException;
}
