/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * --------------------------------------------------------------------------
 * $Id: Properties2Configuration.java,v 1.1.1.1 2002/08/06 13:13:47 bmi Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonathan.apis.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import java.util.Properties;

public interface Properties2Configuration {

   /** 
    * Imports the provided properties into the <code>_context</code>.
    * <p>
    * Property keys are mapped to names in the provided context, using '.'
    * as a separator.
    * <p>
    * Property values are mapped to values of type string, except when the 
    * value may be parsed as:
    * <ul>
    * <li> an integer, in which case it is considered as an int;
    * <li> an boolean, in which case it is considered as an Boolean;
    * <li> a float, in which case it is considered as an Float;
    * </ul>
    * <p> 
    * <code>except</code> specifies a list of property keys prefixes 
    * corresponding to keys that should not be imported into the context.
    *
    * @param _context the context to import the properties to;
    * @param _props   the properties to import;
    * @param except   prefix key prefixes corresponding to properties that should not
    *                be imported.
    * @param force indicates whether already existing elements should be
    *              replaced or not.
    */
   public void importProperties(Context _configuration, Properties _props, String[] _except,
                                boolean _force)
      throws JonathanException;

   /** 
    * Imports the provided property into the <code>_context</code>.
    *
    * @param _context the context to import the property to;
    * @param _name the property name
    * @param _value the property value
    * @param force indicates whether an already existing element should be
    *              replaced or not.
    */
   public void importProperty(Context _context, String _name, String _value, boolean _force)
      throws JonathanException;
   
}
   
