/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 * --------------------------------------------------------------------------
 * $Id: JComponent.java,v 1.1.1.1 2002/08/06 13:13:49 bmi Exp $
 * -------------------------------------------------------------------------- 
 */

package org.objectweb.jonathan.libs.kernel;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.Name;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.helpers.NameHelpers;

/**
 * Basic implementation of a runtime component. Its behaviour
 * is a property component's. Must be subtyped to behave
 * differently.
 */
public class JComponent implements Forkable {   

   /** the type */
   protected Class type;

   /** the object reference value */
   protected Object value;

   /** the integer value */
   protected int int_value;

   /** The element referencing this component */
   protected Element reference;
   
   /** 
    * Constructs a JComponent.
    */
   public JComponent() {
      this(null,Context.NO_VALUE,Integer.MAX_VALUE,null);
   }
      
   public JComponent(Class _type,int _int_value) {
      this(_type,new Integer(_int_value),_int_value,null);
   }
      
   public JComponent(Class _type,Object _value) {
      this(_type,_value,Integer.MAX_VALUE,null);
   }
      
      
   public JComponent(Class _type,Object _value,int _int_value) {
      this(_type,_value,_int_value,null);
   }

   public JComponent(Class _type,Object _value,int _int_value,
                     Element _reference) {
      reference = _reference;
      type = _type;
      value = _value;
      int_value = _int_value;
   }      
      
   public Class getType(){return type;}
   public Object getValue(){return value;}
   public int getIntValue(){return int_value;}
   public Object getFactoryValue() throws JonathanException{
      throw new JonathanException(this + " is not a factory.");
   }
   
   public Element getReference() {
      return reference;
   }
   public Element setReference(Element _reference) {
      Element $former = reference;
      reference = _reference;
      return $former;
   }
   public String toString() {
      return "JComponent[" + type + " " + value + " " + int_value + "]";
   }

   protected Component duplicate() {
      return new JComponent(type,value,int_value,null);
   }

   public Component fork() {
      return internalFork();
   }

   public Component internalFork() {
      Component result = duplicate();
      result.setReference(reference);
      return result;
   }

   public Element getElement(Name name) {
      return null;
   }
 
   final public Component getComponent(Name _name) {
      Element $element = getElement(_name);
      if ($element != null) {
         return $element.getComponent();
      } else {
         return null;
      }
   }
      
   final public Object getValue(Name _name) {
      Element element = getElement(_name);
      if (element != null) {
         return element.getValue();
      } else {
         return Context.NO_VALUE;
      }
   }

   protected void reset() {
      type = null;
      value = null;
   }
}   
