/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 * --------------------------------------------------------------------------
 * $Id: JElement.java,v 1.1.1.1 2002/08/06 13:13:50 bmi Exp $
 * -------------------------------------------------------------------------- 
 */

package org.objectweb.jonathan.libs.kernel;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.Component;


/**
 * Basic implementation of a runtime element. Its behaviour
 * is a property component's. Must be subtyped to behave
 * differently.
 */
public class JElement implements Element {

   Component container;
   String name; 
   Component component;
    
   JElement() {}
   
   public JElement(Component _container, String _name,Component _target) {
      name  = _name;
      component = _target;
      container = _container;
   }
   
   public String getName() {
      return name;
   }
      
   public Component  getComponent() {
      return component;
   }

   public Class getType(){
      return component != null ? component.getType() : null;
   }
   
   public Object getValue(){
      return component != null ? component.getValue() : Context.NO_VALUE;
   }
   
   public int getIntValue(){
      return component != null ? component.getIntValue() : Integer.MAX_VALUE;
   }
   
   public Component getContainer() {
      return container;
   }  

   public String toString() {
      return "Element[" + name + " " + component + "]";
   }
}
