/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $Id: JAlternative.java,v 1.1.1.1 2002/08/06 13:13:51 bmi Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import java.util.List;
import java.util.LinkedList;
/**
 * An implementation alternative's runtime representation. Identifies
 * a single constructor method by handling its class and its arguments
 * types list (as a {@link JArgument} list). Used by its
 * ImplicitFactory to instantiate an implementation.<P>
 * A <code>JAlternative</code> is created along with its
 * ImplicitFactory, either by the application compiled
 * configuration or by the runtime {@link XMLImplicitFactoryDecoder}
 * (if the application's XML configuration has not been compiled, but
 * is used as the XML runtime configuration file instead).
 */
public class JAlternative {
   public String implementation_class;
   public List arguments;
   public List setters;
   public int id;
   public String construction_name;// != null only if FactoryMethod
   public Constructor constructor = null;// set by JImplicitFactory (IF)
   public Method mconstructor = null;// set by JAtom and JAssemblage (FM)
   

   /** 
    * In order to be compliant with old versions.
    * @param _implementation_class the alternative's implementation class
    * @param _arguments list of contruction arguments
    * @param _id the alternative's identifier.
    */
   public JAlternative(String _implementation_class,List _arguments,int _id) {
      this(null,_implementation_class,_arguments,new LinkedList(),_id);
   }

   /** 
    * In order to be compliant with old versions.
    * @param _implementation_class the alternative's implementation class
    * @param _arguments list of contruction arguments
    * @param _setters list of initialisation arguments
    * @param _id the alternative's identifier.
    */
   public JAlternative(String _implementation_class,List _arguments,List _setters,int _id) {
      this(null,_implementation_class,_arguments,_setters,_id);
   }
   
   /**
    * creates the runtime implementation alternative whose implementation
    * class is <code>_implementation_class</code> and whose arguments' types
    * are listed as {@link JArgument} in <code>_arguments</code>. If
    * <code>_name</code> is null, the corresponding constructor is
    * called. Otherwise, the method <code>_name</code> is called. Parameters'
    * for initialisation are stored as {@link JSetter} in <code>_setters</code>.
    */
   public JAlternative(String _name,String _implementation_class,List _arguments,List _setters,int _id) {
      implementation_class = _implementation_class;
      construction_name = _name;
      arguments = _arguments;
      setters = _setters;
      id = _id;
   }
      
   public String toString() {
      if (mconstructor == null){
         return implementation_class + arguments;
      } else {
         return implementation_class + mconstructor + arguments;
      }
   }
}
   
      
