/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Christophe Hartwig
 *
 * -------------------------------------------------------------------
 * $Id: JSequence.java,v 1.1.1.1 2002/08/06 13:13:52 bmi Exp $
 * -------------------------------------------------------------------
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;
import org.objectweb.jonathan.apis.kernel.ClassPathFinder;
import org.objectweb.jonathan.apis.kernel.Name;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.libs.kernel.JTreeElement;
import org.objectweb.jonathan.libs.kernel.decoders.LazyComponent;

import org.objectweb.jonathan.libs.helpers.TraceHelpers;
import org.objectweb.jonathan.libs.helpers.NameHelpers;
import org.objectweb.jonathan.libs.kernel.ClassPathFinderImpl;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.lang.reflect.Array;

/**
 * The atom runtime component. Initialisation issues
 * are dealt with by the {@link LazyComponent} implementation.
 */
public class JSequence extends LazyComponent {
   static final Name SELF_NAME = NameHelpers.newName(".",(char) 0);
   static final Name ROOT_NAME = NameHelpers.newName("",(char) 0);

   protected Element self;

   String elements_class;
   Class elements_class_object=null;
   Vector elements= new Vector();

   public JSequence(String _elements_class, Enumeration _elements) {
      super();
      elements_class = _elements_class;
      while (_elements.hasMoreElements()) {
         elements.addElement(_elements.nextElement());
      }
      self = new JTreeElement(this,".",this);
      for (int $count=0; $count<elements.size(); $count++) {
         Component $component = (Component) elements.elementAt($count);
         Element $element = new JTreeElement(this,Integer.toString($count),$component);
         $component.setReference($element);
      }
   }

   public Component duplicate() {
      return new JSequence(elements_class, elements.elements());
   }

   public String toString() {
      if (! initialized) {
         return "JSequence[" + elements_class + "[]]";
      } else {
         return "JSequence[" + type + " " + value + "]";
      }            
   }

   /**
    * JSequence specific initialization code. Instantiates the JSequence's
    * own value. Used internally by {@link LazyComponent}.
    */
   final protected void initialize() {
      if (! initialized) {
         Component context = reference.getContainer();
            
         Object $component = context.getValue(NameHelpers.newName("/classpath_finder",'/'));
         ClassPathFinder $class_finder = null;
            
         if ($component instanceof ClassPathFinder) {
            $class_finder = (ClassPathFinder) $component;
            type = $class_finder.findClass(getArrayClassName(elements_class));
            if (isPrimitiveType(elements_class)) {
               elements_class_object = (Class)PRIM_TYPES.get(elements_class);
            } else {
               elements_class_object = $class_finder.findClass(elements_class);
            }
         } else {
            type = ClassPathFinderImpl.loadSystemClass(getArrayClassName(elements_class));
            if (isPrimitiveType(elements_class)) {
               elements_class_object = (Class)PRIM_TYPES.get(elements_class);
            } else {
               elements_class_object = ClassPathFinderImpl.loadSystemClass(elements_class);
            }
         }
            
         if (type == null) {
            System.err.println("WARNING in sequence [" +
                               reference +
                               "] : could not instantiate sequence of " +
                               elements_class );
            initialized = true;
            return;
         }
            
         try {
            if (value == Context.NO_VALUE) value = Array.newInstance(elements_class_object, elements.size());

            initialized = true;
            for (int $count=0; $count<elements.size(); $count++) {
               this.setElement(value, $count, (Component)elements.elementAt($count));
            }
            return;

         } catch (ExceptionInInitializerError e2){
            System.err.println("WARNING in sequence [" +
                               reference +
                               "] : could not instantiate sequence "+
                               type + ": " +
                               TraceHelpers.convertStackToString(e2));
            initialized = true;
            return;
         } catch (SecurityException e3){
            System.err.println("WARNING in sequence [" +
                               reference +
                               "] : could not instantiate sequence "+
                               type + ": " +
                               TraceHelpers.convertStackToString(e3));
            initialized = true;
            return;
         }
      }
   }

   public Element getElement(Name _name) {
     String $id  = _name.getId();
      Name $subname = _name.getSubName();
      Element $local = getLocalElement($id);
      if ($local == null || $subname == null) {
         return $local;
      } else {
         Component $local_component = $local.getComponent();
         return $local_component != null ? $local_component.getElement($subname) : null;
      }
   }

   Element getLocalElement(String _id) {
      //System.err.println("Searching \"" + _id + "\" in " + this);
      if (".".equals(_id)) {
         return self; 
      } else if ("..".equals(_id)) {
         return reference != null ? reference.getContainer().getElement(SELF_NAME) : null;
      } else if ("".equals(_id)) {
         if (reference != null) {
            return reference.getContainer().getElement(ROOT_NAME);
         } else {
            return self;
         }
      } else {
         try {
            int $index = Integer.valueOf(_id).intValue();
            Component $comp = (Component)elements.elementAt($index);
            Element $to_return = $comp.getReference();
            return $to_return;
         } catch (NumberFormatException nfex) {
            return null;
         } catch (ArrayIndexOutOfBoundsException aioobex) {
            return null;
         }
      }
   }
   
   private void setElement(Object _array, int _index, Component _comp) {
      if (!isPrimitiveType(elements_class)) {
         Object $val = _comp.getValue();
         //System.err.println("        value is " + $val);
         Array.set(_array, _index, $val);
      } else if ("int".equals(elements_class)) {
         Array.setInt(_array, _index, _comp.getIntValue());
      } else if ("char".equals(elements_class)) {
         Array.setChar(_array, _index,
                       ((Character)_comp.getValue()).charValue());
      } else if ("double".equals(elements_class)) {
         Array.setDouble(_array, _index,
                       ((Double)_comp.getValue()).doubleValue());
      } else if ("float".equals(elements_class)) {
         Array.setFloat(_array, _index,
                       ((Float)_comp.getValue()).floatValue());
      } else if ("byte".equals(elements_class)) {
         Array.setByte(_array, _index,
                       ((Byte)_comp.getValue()).byteValue());
      } else if ("long".equals(elements_class)) {
         Array.setLong(_array, _index,
                       ((Long)_comp.getValue()).longValue());
      } else {
         throw new RuntimeException("unsupported array type " +
                                    elements_class);
      }
   }
   

   private String getArrayClassName(String _class_name) {

      String $try_prim_array = (String)ARRAY_TYPES.get(_class_name);
      if ($try_prim_array == null) { // not a primitive array
         return "[L"+_class_name+";";
      } else {
         return $try_prim_array;
      }
   }

   private boolean isPrimitiveType(String _type) {
      if (PRIM_TYPES.get(_type) == null) { // not found = not prim
         return false;
      } else {
         return true;
      }
   }

   final static Hashtable PRIM_TYPES = new Hashtable();

   final static Hashtable ARRAY_TYPES = new Hashtable();

   static {
      PRIM_TYPES.put("int",Integer.TYPE);
      PRIM_TYPES.put("char",Character.TYPE);
      PRIM_TYPES.put("double",Double.TYPE);
      PRIM_TYPES.put("float",Float.TYPE);
      PRIM_TYPES.put("byte",Byte.TYPE);
      PRIM_TYPES.put("boolean",Boolean.TYPE);
      PRIM_TYPES.put("short",Short.TYPE);
      PRIM_TYPES.put("long",Long.TYPE);

      // not really usefull to create the array, but provides the offered type
      ARRAY_TYPES.put("int","[I");
      ARRAY_TYPES.put("char","[C");
      ARRAY_TYPES.put("double","[D");
      ARRAY_TYPES.put("float","[F");
      ARRAY_TYPES.put("byte","[B");
      ARRAY_TYPES.put("boolean","[Z");
      ARRAY_TYPES.put("short","[S");
      ARRAY_TYPES.put("long","[J");
   }
}
