/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Christophe Hartwig
 *
 * -------------------------------------------------------------------
 * $Id: XMLSequenceDecoder.java,v 1.1.1.1 2002/08/06 13:13:55 bmi Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.kernel.JContextFactory;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoder;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoderDispatcher;

import nanoxml.XMLElement;

import java.util.Enumeration;
import java.util.List;
import java.util.LinkedList;
import java.util.Vector;

/**
 * Imports a runtime sequence component from its XML description.
 * See the XML DTD file. Allow an XMLOpenDecoder to decode
 * an XML configuration file containing valid implicit factory tags.
 * It must therefore be provided in the Context parameter of the
 * <code>OpenXMLDecoder</code>as the value of an element named
 * "IMPLICIT_FACTORY".
 * @see org.objectweb.jonathan.libs.kernel.decoders.JImplicitFactory
 * @see org.objectweb.jonathan.libs.kernel.decoders.OpenXMLDecoder
 */
public class XMLSequenceDecoder implements XMLElementDecoder {
   
   /**
    * returns a new runtime ImplicitFactory component from
    * its NanoXML element description.
    * @param _xml_element its NanoXML description
    * @param _element_name its element's name
    * @param _configuration unused here
    * @return the corresponding runtime implicit factory component
    */
   public Component decode(XMLElement _xml_element, String _element_name, Context _configuration,
                           XMLElementDecoderDispatcher _dispatcher) 
      throws JonathanException {
      try {
         
         JSequence seq = null;

         String $current_tag = _xml_element.getName();

         if ($current_tag.equals("SEQUENCE")) {

            // get the elements type
            String $elements_type = _xml_element.getStringAttribute("ELEMENTS_TYPE");
            if ($elements_type == null) {
               return _dispatcher.error("XMLSequenceDecoder error: missing ELEMENTS_TYPE property " + $current_tag);
            }
         
            // this context will contain the elements contained in the sequence
            // it is necessary to get a context, since decode wants one !
            Vector $sequence_elements = new Vector();

            // this will contain children
            Enumeration $members = _xml_element.enumerateChildren();

            int $child_counter = 0;
            while ($members.hasMoreElements()) {
               XMLElement $child = (XMLElement) $members.nextElement();
               String $elem_tag = $child.getName();
               if ("ELEM".equals($elem_tag)) {
                  Enumeration $actual_component_enum = $child.enumerateChildren();
                  if (!$actual_component_enum.hasMoreElements()) {
                     return _dispatcher.error("XMLSequenceDecoder error: No component found in ELEM");
                  }
                  XMLElement $actual_component = (XMLElement)$actual_component_enum.nextElement();
                  Component $each = _dispatcher.decode($actual_component, null, null);
                  $sequence_elements.addElement($each);
               } else if ("DOCUMENTATION".equals($elem_tag)) {
                  // it's ok, keep cool
               } else {
                  return _dispatcher.error("XMLSequenceDecoder error: contained elements must start with ELEM tag");
               }
            }

            // because the runtime expects a primitive type or class name
            if ("String".equals($elements_type)) {
               $elements_type = "java.lang.String";
            }

            return new JSequence($elements_type,$sequence_elements.elements());
         } else {
            return _dispatcher.error("XMLSequenceDecoder error: Wrong tag " + $current_tag);
         }
      } catch (NullPointerException ex) {
         ex.printStackTrace();
         throw ex;
      }
   }
}
